# react-hooks-global-state

[![CI](https://img.shields.io/github/workflow/status/dai-shi/react-hooks-global-state/CI)](https://github.com/dai-shi/react-hooks-global-state/actions?query=workflow%3ACI)
[![npm](https://img.shields.io/npm/v/react-hooks-global-state)](https://www.npmjs.com/package/react-hooks-global-state)
[![size](https://img.shields.io/bundlephobia/minzip/react-hooks-global-state)](https://bundlephobia.com/result?p=react-hooks-global-state)

Simple global state for React with Hooks API

## Introduction

This is a library to provide a global state with React Hooks.
It has following characteristics.

-   React Hooks only API without React Context
    -   You don't need Context for global state.
-   Hardly misusable selector by state keys
    -   For most of cases, function selectors are not necessary.
-   TypeScript type definitions
    -   A creator function creates hooks with types inferred.
-   Analogous APIs
    -   useGlobalState works like React.useState.
    -   useAtom resembles with the concept of Recoil.
    -   createStore implies Redux.
-   Concurrent Mode support
    -   Under the hood, it's implemented with useMutableSource.
    -   (For external stores like this, state branching is never possible.)

## Install

```bash
npm install react-hooks-global-state
```

## Usage

### createAtom style

```javascript
import React from 'react';
import { createAtom, useAtom } from 'react-hooks-global-state';

const initialState = { count: 0 };
const atom1 = createAtom(initialState);

const Counter = () => {
  const [count, setCount] = useAtom(atom1, 'count');
  return (
    <div>
      <span>Counter: {count}</span>
      {/* update state by passing callback function */}
      <button onClick={() => setCount(v => v + 1)}>+1</button>
      {/* update state by passing new value */}
      <button onClick={() => setCount(count - 1)}>-1</button>
    </div>
  );
};

const App = () => (
  <>
    <Counter />
    <Counter />
  </>
);
```

### createGlobalState style

```javascript
import React from 'react';
import { createGlobalState } from 'react-hooks-global-state';

const initialState = { count: 0 };
const { useGlobalState } = createGlobalState(initialState);

const Counter = () => {
  const [count, setCount] = useGlobalState('count');
  return (
    <div>
      <span>Counter: {count}</span>
      {/* update state by passing callback function */}
      <button onClick={() => setCount(v => v + 1)}>+1</button>
      {/* update state by passing new value */}
      <button onClick={() => setCount(count - 1)}>-1</button>
    </div>
  );
};

const App = () => (
  <>
    <Counter />
    <Counter />
  </>
);
```

### createStore style

```javascript
import React from 'react';
import { createStore } from 'react-hooks-global-state';

const reducer = (state, action) => {
  switch (action.type) {
    case 'increment': return { ...state, count: state.count + 1 };
    case 'decrement': return { ...state, count: state.count - 1 };
    default: return state;
  }
};
const initialState = { count: 0 };
const { dispatch, useGlobalState } = createStore(reducer, initialState);

const Counter = () => {
  const [value] = useGlobalState('count');
  return (
    <div>
      <span>Counter: {value}</span>
      <button onClick={() => dispatch({ type: 'increment' })}>+1</button>
      <button onClick={() => dispatch({ type: 'decrement' })}>-1</button>
    </div>
  );
};

const App = () => (
  <>
    <Counter />
    <Counter />
  </>
);
```

## API

<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

### createAtom

create an atom

It returns a set of functions to be called outside React

-   `getStateByKey`: a function to get the part of state by key outside React
-   `setStateByKey`: a function to set the part of state by key outside React
-   `getState`: a function to get the entire state
-   `setState`: a function to get the entire state
-   `dispatch`: an optional function that can be used if reducer is provided

#### Parameters

-   `initialState` **State** 
-   `reducer` **Reducer&lt;State, Action>?** 

#### Examples

```javascript
import { createAtom } from 'react-hooks-global-state';

const atom = createAtom({ count: 0 });

atom.setStateByKey('count', 1);
```

### createGlobalState

create a gloal state

It returns a set of functions

-   `useGlobalState`: a custom hook works like React.useState
-   `getGlobalState`: a function to get a global state by key outside React
-   `setGlobalState`: a function to set a global state by key outside React

#### Parameters

-   `initialState` **State** 

#### Examples

```javascript
import { createGlobalState } from 'react-hooks-global-state';

const { useGlobalState } = createGlobalState({ count: 0 });

const Component = () => {
  const [count, setCount] = useGlobalState('count');
  ...
};
```

### createStore

create a global store

In additon to `useGlobalState` which is the same hook as in createGlobalState,
a store has `getState` and `dispatch`.
A store works somewhat similarly to Redux, but not the same.

#### Parameters

-   `reducer` **Reducer&lt;State, Action>** 
-   `initialState` **State**  (optional, default `(reducer as any)(undefined,{type:undefined})`)
-   `enhancer` **Enhancer&lt;any>?** 

#### Examples

```javascript
import { createStore } from 'react-hooks-global-state';

const initialState = { count: 0 };
const reducer = ...;

const store = createStore(reducer, initialState);
const { useGlobalState } = store;

const Component = () => {
  const [count, setCount] = useGlobalState('count');
  ...
};
```

### useAtom

use atom

a custom hook that works like React.useState

#### Parameters

-   `atom` **Atom&lt;State, any>** 
-   `stateKey` **any?** 

#### Examples

```javascript
import { createAtom, useAtom } from 'react-hooks-global-state';

const atom = createAtom({ count: 0 });

const Component = () => {
  const [count, setCount] = useAtom(atom, 'count');
  ...
};
```

## Examples

The [examples](examples) folder contains working examples.
You can run one of them with

```bash
PORT=8080 npm run examples:01_minimal
```

and open <http://localhost:8080> in your web browser.

You can also try them in codesandbox.io:
[01](https://codesandbox.io/s/github/dai-shi/react-hooks-global-state/tree/master/examples/01_minimal)
[02](https://codesandbox.io/s/github/dai-shi/react-hooks-global-state/tree/master/examples/02_typescript)
[03](https://codesandbox.io/s/github/dai-shi/react-hooks-global-state/tree/master/examples/03_actions)
[04](https://codesandbox.io/s/github/dai-shi/react-hooks-global-state/tree/master/examples/04_fetch)
[05](https://codesandbox.io/s/github/dai-shi/react-hooks-global-state/tree/master/examples/05_onmount)
[06](https://codesandbox.io/s/github/dai-shi/react-hooks-global-state/tree/master/examples/06_reducer)
[07](https://codesandbox.io/s/github/dai-shi/react-hooks-global-state/tree/master/examples/07_middleware)
[08](https://codesandbox.io/s/github/dai-shi/react-hooks-global-state/tree/master/examples/08_thunk)
[09](https://codesandbox.io/s/github/dai-shi/react-hooks-global-state/tree/master/examples/09_comparison)
[10](https://codesandbox.io/s/github/dai-shi/react-hooks-global-state/tree/master/examples/10_immer)
[11](https://codesandbox.io/s/github/dai-shi/react-hooks-global-state/tree/master/examples/11_deep)
[12](https://codesandbox.io/s/github/dai-shi/react-hooks-global-state/tree/master/examples/12_effect)
[13](https://codesandbox.io/s/github/dai-shi/react-hooks-global-state/tree/master/examples/13_persistence)

## Blogs

-   [TypeScript-aware React hooks for global state](https://blog.axlight.com/posts/typescript-aware-react-hooks-for-global-state/)
-   [An alternative to React Redux by React Hooks API (For both JavaScript and TypeScript)](https://blog.axlight.com/posts/an-alternative-to-react-redux-by-react-hooks-api-for-both-javascript-and-typescript/)
-   [Redux middleware compatible React Hooks library for easy global state management](https://blog.axlight.com/posts/redux-middleware-compatible-react-hooks-library-for-easy-global-state-management/)
-   [React Hooks Tutorial on pure useReducer + useContext for global state like Redux and comparison with react-hooks-global-state](https://blog.axlight.com/posts/react-hooks-tutorial-for-pure-usereducer-usecontext-for-global-state-like-redux-and-comparison/)
-   [Four patterns for global state with React hooks: Context or Redux](https://blog.axlight.com/posts/four-patterns-for-global-state-with-react-hooks-context-or-redux/)
-   [Steps to Develop Global State for React With Hooks Without Context](https://blog.axlight.com/posts/steps-to-develop-global-state-for-react/)

## Community Wiki

-   [Persistence](https://github.com/dai-shi/react-hooks-global-state/wiki/Persistence)
-   [Optional initialState](https://github.com/dai-shi/react-hooks-global-state/wiki/Optional-initialState)
